@@ -51,6 +51,7 @@ const TokenDict SDPParser::s_payloads[] = {
     { "speex/32000", 104 },
     { "isac/16000",  105 },
     { "isac/32000",  106 },
+    { "gsm-efr",     107 },
     { "mjpeg",        26 },
     { "h261",         31 },
     { "h263",         34 },
@@ -82,6 +83,7 @@ const TokenDict SDPParser::s_rtpmap[] = {
     { "SPEEX/32000", 104 },
     { "iSAC/16000",  105 },
     { "iSAC/32000",  106 },
+    { "GSM-EFR/8000",107 },
     { "JPEG/90000",   26 },
     { "H261/90000",   31 },
     { "H263/90000",   34 },
