@@ -406,6 +406,7 @@ static void adjustPath(String& script)
     String tmp = Engine::sharedPath();
     tmp << Engine::pathSeparator() << "scripts";
     tmp = s_cfg.getValue("general","scripts_dir",tmp);
+    Engine::runParams().replaceParams(tmp);
     if (!tmp.endsWith(Engine::pathSeparator()))
 	tmp += Engine::pathSeparator();
     script = tmp + script;
@@ -1965,8 +1966,11 @@ void ExtModulePlugin::initialize()
 	    unsigned int len = sect->length();
 	    for (unsigned int i=0; i<len; i++) {
 		NamedString *n = sect->getParam(i);
-		if (n)
-		    ExtModReceiver::build(n->name(),*n);
+		if (n) {
+		    String arg = *n;
+		    Engine::runParams().replaceParams(arg);
+		    ExtModReceiver::build(n->name(),arg);
+		}
 	    }
 	}
 	// and now start additional programs
@@ -1975,8 +1979,14 @@ void ExtModulePlugin::initialize()
 	    unsigned int len = sect->length();
 	    for (unsigned int i=0; i<len; i++) {
 		NamedString *n = sect->getParam(i);
-		if (n)
-		    runProgram(n->name(),*n);
+		if (n) {
+		    String tmp = n->name();
+		    String arg = *n;
+		    adjustPath(tmp);
+		    Engine::runParams().replaceParams(arg);
+		    if (tmp)
+			runProgram(tmp,arg);
+		}
 	    }
 	}
     }
